#########################################
###  SET UP SHOP FIRST: INSTALL/PREP  ###
#########################################

# Clean up your global environment first
rm(list=ls())

# Set working directory and apply global settings
setwd("~/Desktop/NicheOverlap")
options(stringsAsFactors = F)
pars <- par()

# Consider unloading currently loaded libraries to clear memory
# invisible(unloadNamespace(.packages()))

libraries <- c("rgdal", "raster", "ade4", "adegraphics", "ggplot2",
               "spdep", "maptools", "MASS", "ggord", "gtools", "ecospat")
sapply(libraries, function(x) { suppressMessages(require(x, character.only=T)) } )

# Import files
pops <- read.table("pops.txt", header=T)
p <- pops[,which(names(pops) %in% c("pop", "Long", "Lat"))]
lineage <- pops[,which(names(pops) %in% c("pop", "lineage"))]


########################################
###  EXTRACT ALL ENVIRONMENTAL DATA  ###
########################################

# Extract elevation from DEM
elevation <- raster("./GIS/current_env/elevation.tif")
toad.env.data <- data.frame( pop=p[,1], long=p[,2], lat=p[,3], elevation=extract(elevation, p[,-1]) )

# Extract Bioclim variables
files1 <- list.files(path="./GIS/current_env", pattern='bio_[0-9]+\\.tif$')
files1 <- mixedsort(files1)
files1 <- paste0("./GIS/current_env/", files1)
r1 <- sapply(files1, function(x) { raster(x) } )
s1 <- stack(r1)
names(s1) <- gsub(".tif", "", names(s1))
names(s1) <- gsub("..GIS.current_env.", "", names(s1))
names(s1) <- gsub("_", "", names(s1))
toad.env.data <- data.frame( toad.env.data, extract(s1, p[,-1]) )
toad.env.data <- merge(lineage, toad.env.data)
groups <- toad.env.data[,c(1:4)]

# Resample all rasters to one stack (for prediction/mapping later on)
all.r <- c(elevation, r1)
all.r.res <- list()
for (i in 1:length(all.r)) {
  all.r.res <- c( all.r.res, resample(all.r[[i]], r1[[1]]) ) }
all.s <- stack(all.r.res)
mask <- all.s[[1]]/all.s[[1]] - 1

# Extract data for all pixels in Yosemite
yose <- readOGR(dsn="./GIS", layer="YOSEbndry")
canorus <- readOGR(dsn="./GIS", layer="canorus")
poly1 <- intersect(canorus, yose)
all.env.pred <- data.frame( do.call("rbind", extract(all.s, poly1, cellnumbers=T ) ) )
longlat <- xyFromCell(mask, all.env.pred$cell)
all.env.pred <- data.frame(all.env.pred$cell, longlat, all.env.pred[,-1])
names(all.env.pred)[1:3] <- c("cell", "long", "lat")
cell1 <- all.env.pred$cell
all.env.pred <- all.env.pred[,-c(1:3)]
names(all.env.pred) <- gsub("_", "", names(all.env.pred))

# Linear Discriminant Analysis
toad.lda.data <- toad.env.data[,-c(1,3,4)]
biovars <- c("lineage","bio1","bio3","bio8","bio14","bio15","bio17")

# Remove collinear variables
cl <- toad.lda.data[,which(names(toad.lda.data) %in% biovars)]

# The actual LDA
lda.fit <- lda(lineage ~ ., data = cl)
print(lda.fit)
lda.predict <- predict(object = lda.fit, newdata = cl)

# Plot the LDA results
Lineages <- factor(lineage$lineage, levels = c("North","East-North-H1",
                                               "East-North-H2","East",
                                               "East-South-H", "South","West"))
colors <- c("#FF3939","#F7B5E9","#C500FF","#0070FF","#00FFE5","#4CE600","black")

if( !file.exists("plots") ) {
  dir.create("plots")
}

pdf("./plots/LDA.pdf", useDingbats=F)
ggord(lda.fit, Lineages, poly=F, vec_ext = 2.5, ext = 1.35, size = 2.0) +
  scale_colour_manual(values=colors) +
  coord_fixed(ratio = 2.0) 
dev.off()



# Calculate pairwise niche overlap
yose.pca.env <- dudi.pca(all.env.pred,scannf=F,nf=2) 

# Do background env PCAs for each region separately... this takes a sec
# NORTH
n.pts <- p[which(lineage$lineage == "North"),-1]
n.pts <- n.pts[c(chull(n.pts),chull(n.pts)[1]),]
n.pts <- SpatialPolygons(list(Polygons(list(Polygon(n.pts)), ID=1)),
                         proj4string = CRS("+proj=longlat +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +no_defs"))
cell2 <- data.frame( do.call("rbind", extract(all.s, n.pts, cellnumbers=T ) ) )$cell
north.pca.env <- yose.pca.env$li[which(cell1 %in% cell2),]
n.coords <- p[which(lineage$lineage == "North"),-1]
cell3 <- data.frame(extract(all.s, n.coords, cellnumbers=T))$cells
north.pca.pt <- yose.pca.env$li[which(cell1 %in% cell3),]

# EAST
e.pts <- p[which(lineage$lineage == "East"),-1]
e.pts <- e.pts[c(chull(e.pts),chull(e.pts)[1]),]
e.pts <- SpatialPolygons(list(Polygons(list(Polygon(e.pts)), ID=1)),
                         proj4string = CRS("+proj=longlat +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +no_defs"))
cell2 <- data.frame( do.call("rbind", extract(all.s, e.pts, cellnumbers=T ) ) )$cell
east.pca.env <- yose.pca.env$li[which(cell1 %in% cell2),]
e.coords <- p[which(lineage$lineage == "East"),-1]
cell3 <- data.frame(extract(all.s, e.coords, cellnumbers=T))$cells
east.pca.pt <- yose.pca.env$li[which(cell1 %in% cell3),]

# SOUTH
s.pts <- p[which(lineage$lineage == "South"),-1]
s.pts <- s.pts[c(chull(s.pts),chull(s.pts)[1]),]
s.pts <- SpatialPolygons(list(Polygons(list(Polygon(s.pts)), ID=1)),
                         proj4string = CRS("+proj=longlat +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +no_defs"))
cell2 <- data.frame( do.call("rbind", extract(all.s, s.pts, cellnumbers=T ) ) )$cell
south.pca.env <- yose.pca.env$li[which(cell1 %in% cell2),]
s.coords <- p[which(lineage$lineage == "South"),-1]
cell3 <- data.frame(extract(all.s, s.coords, cellnumbers=T))$cells
south.pca.pt <- yose.pca.env$li[which(cell1 %in% cell3),]

# WEST
w.pts <- p[which(lineage$lineage == "West"),-1]
w.pts <- w.pts[c(chull(w.pts),chull(w.pts)[1]),]
w.pts <- SpatialPolygons(list(Polygons(list(Polygon(w.pts)), ID=1)),
                         proj4string = CRS("+proj=longlat +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +no_defs"))
cell2 <- data.frame( do.call("rbind", extract(all.s, w.pts, cellnumbers=T ) ) )$cell
west.pca.env <- yose.pca.env$li[which(cell1 %in% cell2),]
w.coords <- p[which(lineage$lineage == "West"),-1]
cell3 <- data.frame(extract(all.s, w.coords, cellnumbers=T))$cells
west.pca.pt <- yose.pca.env$li[which(cell1 %in% cell3),]


clim <- list()

# gridding the north niche
grid.clim.north <- ecospat.grid.clim.dyn(glob=yose.pca.env$li, 
                                       glob1=yose.pca.env$li,
                                       sp=north.pca.pt, R=200,
                                       th.sp=0.05)
clim[["North"]] <- grid.clim.north

# gridding the east niche
grid.clim.east <- ecospat.grid.clim.dyn(glob=yose.pca.env$li,
                                         glob1=yose.pca.env$li,
                                         sp=east.pca.pt, R=200,
                                         th.sp=0.05)
clim[["East"]] <- grid.clim.east

# gridding the south niche
grid.clim.south <- ecospat.grid.clim.dyn(glob=yose.pca.env$li, 
                                         glob1=yose.pca.env$li,
                                         sp=south.pca.pt, R=200,
                                         th.sp=0.05)
clim[["South"]] <- grid.clim.south

# gridding the west niche
grid.clim.west <- ecospat.grid.clim.dyn(glob=yose.pca.env$li, 
                                         glob1=yose.pca.env$li,
                                         sp=west.pca.env, R=200,
                                         th.sp=0.05)
clim[["West"]] <- grid.clim.west

# Compute Schoener's D, index of niche overlap
temp.list <- list()
mat <- matrix(nrow=length(clim), ncol=length(clim))
sig <- matrix(nrow=length(clim), ncol=length(clim))
row.names(mat) <- row.names(sig) <- names(clim)
colnames(mat) <- colnames(sig) <- names(clim)
for(i in 1:length(clim)) {
  for(j in 1:length(clim)) {
    if(i < j) {
      D.overlap <- ecospat.niche.overlap(clim[[i]], clim[[j]], cor=T)$D 
      mat[i,j] <- D.overlap
      
      N.equiv <- ecospat.niche.equivalency.test(clim[[i]], clim[[j]], rep=1000, alternative="lower")
      
      # If the function screws up and returns some NAs for p-values, recalculate manually...
      if (!is.na( N.equiv$p.D)) {
        sig[i,j] <- N.equiv$p.D
      } else {
        temp.list[[paste0(names(clim)[i], ".", names(clim)[j])]] <- N.equiv
        
        sim <- N.equiv$sim$D
        sim <- sim[!is.na(sim)]
        obs <- N.equiv$obs$D
        sig[i,j] <- (sum(sim <= obs) + 1)/(length(sim) + 1)
        
      }
    } else {
      mat[i,j] <- NA
      sig[i,j] <- NA
    }
  }
}

round(mat, 3)
round(sig, 3)


